(in-package :remote-snake-server-api)

(defclass api ()
  ((gm
    :initform (make-instance 'game-manager))))

;;; Parse the request and return it as a plist
(defun parse-request (request)
  (flet ((normalizer (key) (string-upcase key)))
        (let* ((p-request (parse request :normalize-all t :keyword-normalizer #'normalizer ))
               (type (getf p-request :type :not-found)))
          (cond
           ((eq type :not-found)
            (error "Invalid request: Bad request type"))
           ((equal type "update")
            (progn
              (unless (getf p-request :game-id) (error "Invalid request: No game id"))
              (let ((dir (getf p-request :direction :not-found)))
                (when (eq :not-found dir) (error "Invalid request: No snake direction provided"))
                (unless (or (equal "up" dir) (equal "down" dir) (equal "left" dir) (equal "right" dir) (eq nil dir)) (error "Invalid request: Bad direction"))
                (cond
                 ((equal dir "up") (setf (getf p-request :direction) :up))
                 ((equal dir "down") (setf (getf p-request :direction) :down))
                 ((equal dir "left") (setf (getf p-request :direction) :left))
                 ((equal dir "right") (setf (getf p-request :direction) :right))))))
           ((not (equal type "new-game"))
            (error "Invalid request: Unknow request type")))
          p-request)))
               
          

(defmethod handle-new-game ((api api) data)
  (with-slots (gm) api
    (let* ((game-id (create-game gm)))
      (let ((game-dump (dump gm game-id)))
        (setf (getf game-dump :game-over) :null) ; Define nil as null (for json)
      (to-json
       (append (list :type "state") game-dump))))))

(defmethod handle-update ((api api) data)
  (with-slots (gm) api
    (let* ((dir (getf data :direction))
          (game-id (getf data :game-id))
          (game (get-game gm game-id)))
      (cond
       ((equal dir "up") (setf dir :up))
       ((equal dir "down") (setf dir :down))
       ((equal dir "left") (setf dir :left))
       ((equal dir "right") (setf dir :right))
       (t (setf dir nil)))
      (if dir
          (refresh game :dir dir)
        (refresh game))
      (to-json
       (append (list :type "state") (dump gm game-id))))))


;;; TODO: Improve error management
(defmethod handle-request ((api api) request)
  (flet ((handle-fun ()
                     (let* ((data (parse-request request))
                            (type (getf data :type)))
                       (cond
                        ((equal type "new-game") (handle-new-game api data))
                        ((equal type "update") (handle-update api data))
                        (t (format t "Unknow type"))))))
        
        (handler-case
            (handle-fun)
          (t (c)
             (format t "Got an exception: ~a~%" c)
             "Bad request"))))



